{% extends "sysscope/base.html" %}
{% load i18n %}
{% load sysscope_tags %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
        &rsaquo; {% trans "Welcome to SysScope" %}
    </div>
{% endblock %}

{% block title %}{% trans "SysScope - Welcome" %}{% endblock %}

{% block sysscope_sidebar %}
    {% graph_section_links CFG start timeframe %}
{% endblock %}

{% block sysscope_main %}
    <h2>{% trans "Welcome to SysScope" %}</h2>
    <p>{% trans "Use the menu on the left to browse through the graphs." %}</p>
{% endblock %}
