{% load i18n %}

<div id="graph_sections">
    <ul>
        {% for section_obj in CFG.sections_ordered %} 
            <li><a href="{% url 'section' section=section_obj.canonical_name start=start timeframe=timeframe %}">{{ section_obj.name }}</a></li>
        {% endfor %}
    </ul>
</div> <!-- graph_sections -->
