{% extends "admin/base_site.html" %}
{% load i18n static %}

{% block extrastyle %}
<link rel="stylesheet" type="text/css" href="{% static "sysscope/css/style.css" %}" />
{% endblock %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
        &rsaquo; {% trans "SysScope" %}
    </div>
{% endblock %}

{% block title %}{% trans "SysScope" %} - {{ title }}{% endblock %}

{% block content %}

    <div id="sysscope-container">
    
	    <div id="sysscope-header">
	        <h1><a href="{% url 'start' %}">SysScope</a></h1>
	        {# <p>{% trans 'Use the menu on the left to browse through the graphs.' %}</p> #}
	    </div> <!-- sysscope-header -->
    
        <div id="sysscope-navigation">
            {% block sysscope_navigation %}{% endblock %}
        </div> <!-- sysscope-navigation -->
        
        <div class="clear"></div>
        
        <div id="sysscope-sidebar">
            {% block sysscope_sidebar %}{% endblock %}
        </div> <!-- sysscope-sidebar -->
        
        <div id="sysscope-main">
            {% block sysscope_main %}{% endblock %}
        </div> <!-- sysscope-main -->
        
        <div class="clear"></div>
        
        <div id="sysscope-footer">
            {% block sysscope_footer %}{% endblock %}
        </div> <!-- sysscope-footer -->
        
{% endblock %}
