{% extends "sysscope/base.html" %}
{% load i18n %}
{% load sysscope_tags %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
        &rsaquo; <a href="{% url 'start' %}">{% trans 'SysScope' %}</a>
        &rsaquo; {% section_report_title section_obj start timeframe %}
    </div>
{% endblock %}

{% block title %}{% trans "SysScope" %} - {% section_report_title section_obj start timeframe %}{% endblock %}

{% block sysscope_navigation %}
    {% nav_timeframe_selector section_obj timeframe %}
    {% nav_timeframe_browser section_obj start timeframe %}
{% endblock %}

{% block sysscope_sidebar %}
    {% graph_section_links CFG start timeframe %}
{% endblock %}

{% block sysscope_main %}
    <h2>{% section_report_title section_obj start timeframe %}</h2>
    <p>{% trans "Report time frame" %}: {{ t_start|date:"D, M j Y H:i:s" }} - {{ t_end|date:"D, M j Y H:i:s" }}</p>
    
    {% for graph_obj in graph_list %}
        <div id="graph-{{ forloop.counter }}" class="graph">
		    <a href="{% url 'graph_archive' section=section_obj.canonical_name graph=graph_obj.CanonicalName %}">
		        <img src="{% url 'graph_image' section=section_obj.canonical_name graph=graph_obj.CanonicalName start=start timeframe=timeframe %}" alt="{{ section_obj.name }} - {{ graph_obj.Name }}" />
		    </a>
		</div> <!-- graph-{{ forloop.counter }} -->
    {% endfor %}
    
{% endblock %}
