
#sysscope-sidebar, #sysscope-main, #sysscope-navigation, #sysscope-footer {
    margin-top: 10px;
}

#sysscope-sidebar {
    float: left;
    width: 300px;
}

#sysscope-main {
    margin-left: 310px;
}

.graph {
    margin-top: 20px;
}

.clear {
    clear: both;
}

#chrono_preset_nav {
    float: left;
}

#timeframe_browser_nav {
    margin-left: 310px;
}
