{% load i18n %}
{% load sysscope_tags %}

<div id="chrono_preset_nav">
    <form id="chrono_preset_form" action="">
        <label for="id_graph_chrono" class="">{% trans 'Select timeframe' %}:</label>
        <select id="id_graph_chrono" name="graph_chrono" onchange="window.location =
            (document.forms.chrono_preset_form.graph_chrono[document.forms.chrono_preset_form.graph_chrono.selectedIndex].value);">
            <option value=''>Select Period</option>
            {% for timeframe in timeframe_list %}
                {% if timeframe == current_timeframe %}
                    <option value="{% url 'section' section=section_obj.canonical_name start=timeframe|negative timeframe=timeframe %}" selected>{{ timeframe|friendlytime }}</option>
                {% else %}
                    <option value="{% url 'section' section=section_obj.canonical_name start=timeframe|negative timeframe=timeframe %}">{{ timeframe|friendlytime }}</option>
                {% endif %}
            {% endfor %}
        </select>
    </form>
</div>