{% extends "sysscope/base.html" %}
{% load i18n %}
{% load sysscope_tags %}

{% block breadcrumbs %}
    <div class="breadcrumbs">
        <a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
        &rsaquo; <a href="{% url 'start' %}">{% trans 'SysScope' %}</a>
        &rsaquo; <a href="{% url 'section' section=section_obj.canonical_name start='-d' timeframe='d' %}">{{ section_obj.name }}</a>
        &rsaquo; {{ graph_obj.Name }} {% trans "archives" %}
    </div>
{% endblock %}

{% block title %}{% trans "SysScope" %} - {{ graph_obj.Name }} {% trans "archives" %}{% endblock %}

{% block sysscope_sidebar %}
    {% graph_archive_links CFG section_obj graph_obj graph_list start timeframe %}
{% endblock %}

{% block sysscope_main %}
    <h2>{{ graph_obj.Name }} {% trans "archives" %}</h2>
    <p>{% trans "This page displays an archive of the selected graph." %}</p>
    
    {% for timeframe in timeframe_list %}
        <div id="graph-{{ forloop.counter }}" class="graph">
            <h3>{{ timeframe|timelyadverb|capfirst }} Graph</h3>
            <img src="{% url 'graph_image' section=section_obj.canonical_name graph=graph_obj.CanonicalName start=timeframe|negative timeframe=timeframe %}" alt="{{ section_obj.name }} - {{ graph_obj.Name }} {{ timeframe|timelyadverb }}" />
		</div> <!-- graph-{{ forloop.counter }} -->
    {% endfor %}
    
{% endblock %}
