{% load i18n %}

<div id="archive_sections">
    <ul>
        <li><a href="{% url 'section' section=section_obj.canonical_name start='-d' timeframe='d' %}"><strong>Up</strong></a></li>
        {% for graph_obj in graph_list %}
            <li>
                <a href="{% url 'graph_archive' section=section_obj.canonical_name graph=graph_obj.CanonicalName %}">{{ graph_obj.Name }}</a>
            </li>
        {% endfor %}
    </ul>
</div> <!-- graph_sections -->
